// Generated by CoffeeScript 2.5.1
(function() {
  /*
  Tree Navigation utility.
  - Allows expanding and collapsing of nodes within a 'tree'
  - Uses Fragmenty to load tree branch data on the fly
  - Tree can exist within a table (e.g. first column has tree entries in to expand and show data in table
  - Tree should be able to take the form of a structured list, but not tested

  Requirements:
  - jQuery
  - Fragmenty (optional, for dynamicly loading branch entries)

  Usage:
  treenav = new TreeNav ({
      debug: 2,
      fragmenty: frag    # Fragmenty object for appending branch data into tree
  });
  */
  window.TreeNav = class TreeNav {
    constructor(opt1) {
      this.findSiblings = this.findSiblings.bind(this);
      this.navigate = this.navigate.bind(this);
      this.showActive = this.showActive.bind(this);
      this.openParent = this.openParent.bind(this);
      this.autoClose = this.autoClose.bind(this);
      this.openclose = this.openclose.bind(this);
      
      // TreeNav function 
      this.actionLevel = this.actionLevel.bind(this);
      this.actionOC = this.actionOC.bind(this);
      this.opt = opt1;
      this._opt = { // set default values
        // instance variables (ref using @_opt.variable_name
        debug: 0, // 0 = no debuggin info, 1 = some debugging info
        activeItemClass: 'is-active',
        autoClose: true, // automatically close other groups when one is opened
        closedHTML: '<i class="twisty-closed-hvr"></i>',
        closedClass: "twisty-closed-hvr",
        openHTML: '<i class="twisty-open-hvr"></i>',
        openClass: "twisty-open-hvr",
        sitePath: null,
        openlevels: 1, // How many levels to expand when opening a node */
        fragmenty: null // Fragmenty object to handle dynamic loading of tree content if neccessary
      };
      $.extend(true, this._opt, this.opt); // overwrite defaults with those provided
      this.init();
    }

    init(opt) {
      var match, site;
      // handle navigation tree actions, incl. links
      // assumes all .treenav containers already exist on the page
      // new content loaded within these .treenav containers will be handled by the event registered here
      // new .treenav containers won't be handled.
      $(document.body).on("click", ".treenode .treeitem", this.openclose);
      if (this._opt.sitePath.startsWith('/')) {
        match = window.location.href.trim().match(/^((http[s]?|ftp):\/\/)?\/?([^\/\s]+)/i);
        site = match[1] + match[3];
        this._opt.debug > 0 && console.log("TreeNav.init site: " + site);
        this._opt.sitePath = site + this._opt.sitePath;
        this._opt.debug > 0 && console.log("TreeNav.init updated sitePath: " + this._opt.sitePath);
      }
      $(window).bind('popstate', this.navigate);
      $(window).bind('fragmenty:load', this.showActive); // Custom event to allow interaction with Fragmenty
      this._opt.debug > 0 && console.log("TreeNav.init");
      return this.showActive();
    }

    findSiblings(el) {
      var next, prev, reflist, siblings;
      // Find those tree rows that are within the same section as this element
      // This does not yet serve a practical purpose, but will be used to load
      // related content (siblings) above and below the main content
      el = $(el).closest('.treerow'); // work at treerow level where relavent data is held
      next = $(el).nextUntil('.treenode');
      prev = $(el).prevUntil('.treenode');
      siblings = prev.add(el).add(next);
      reflist = siblings.map(function() {
        return $(this).attr('-data-ref');
      }).get();
      return this._opt.debug > 1 && console.log('sibling refs: ' + reflist.toString());
    }

    navigate(e) {
      var link;
      // Handle change due to browser navigation buttons (forward and back)
      link = location.href;
      this._opt.debug > 0 && console.log("TreeNav.navigate new location: " + link);
      this._opt.debug > 0 && console.log("TreeNav.navigate sitePath: " + this._opt.sitePath);
      
      // Check if still within the 'master' area we are responsible for
      if (link.startsWith(this._opt.sitePath)) {
        if (!this._opt.fragmenty) {
          return this.showActive(); // if linked to fragmenty, let fragmenty trigger showActive function
        } else {
          return this._opt.debug > 0 && console.log("TreeNav.navigate letting Fragmenty trigger showActive");
        }
      } else {
        return this._opt.debug > 0 && console.log("TreeNav.navigate outside our site, ignoring");
      }
    }

    showActive() {
      var link, query, relative, trimmed;
      // Finds the anchor element which contains the current path and marks it
      // as active using the class 'active'. Also opens up the parent in the 
      // navigation tree to ensure this link item is visible in the navigation tree
      this._opt.debug > 0 && console.log("TreeNav.showActive");
      link = location.href;
      relative = location.href.replace(location.origin, '').replace(location.hash, '').replace(location.search, '').replace(/\/*$/, '');
      this._opt.debug > 1 && console.log("TreeNav.showActive relative: " + relative);
      
      // remove active class from tn-linkitem object not matching current relative href
      $('.tn-linkitem a').not('[href|="' + relative + '"]').closest(".tn-linkitem").removeClass(this._opt.activeItemClass);
      
      // find linkitems that do match the current href (beginning of it)
      query = $('.tn-linkitem a[href|="' + relative + '"]').closest(".tn-linkitem");
      
      // not found try with /index.html added
      if (!$(query).length > 0) { // not found try again with /index.html added to the end
        this._opt.debug > 1 && console.log("TreeNav.showActive looking again: " + relative + '/index.html');
        query = $('.tn-linkitem a[href|="' + relative + '/index.html"]').closest(".tn-linkitem");
      }
      if (!$(query).length > 0) { // not found try trimming the url and looking again
        trimmed = relative.split('/').slice(0, -1).join('/');
        this._opt.debug > 1 && console.log("TreeNav.showActive looking again: " + trimmed);
        query = $('.tn-linkitem a[href|="' + trimmed + '"]').closest(".tn-linkitem");
      }
      while ($(query).length = 0 && (trimmed.split('/').length > 1)) { // not found try trimming the url and looking again
        trimmed = trimmed.split('/').slice(0, -1).join('/');
        this._opt.debug > 1 && console.log("TreeNav.showActive looking again: " + trimmed);
        query = $('.tn-linkitem a[href|="' + trimmed + '"]').closest(".tn-linkitem");
      }
      $(query).addClass(this._opt.activeItemClass);
      this._opt.debug > 1 && console.log("TreeNav.showActive query: " + query.length);
      this._opt.debug > 1 && console.log("TreeNav.showActive about to open parent");
      return this.openParent($(query).first()); // assumes only one TreeNav linkitem will be found for the current address
    }

    openParent(el) {
      var action, openLevel, openList, row, rowId, rowLevel;
      // Get parent of a tree node and set it to open
      // TODO: Start by opening top level node if closed, then move down descendant treenodes in turn
      rowId = $(el).attr("id");
      this._opt.debug > 1 && console.log("TreeNav.openParent of " + rowId);
      // get treerow
      row = $(el).closest('.treerow');
      rowLevel = parseInt($(row).attr('data-tnlevel'));
      if (rowLevel > 0) {
        openLevel = rowLevel - 1; // level we are looking for parent node in
      }
      rowId = $(row).attr("id");
      action = false;
      openList = [];
      while (rowLevel > 0) {
        
        // Find next treenode further up the hierarchy
        row = $(row).prevAll('.treenode[data-tnlevel="' + openLevel + '"]:first');
        rowLevel = parseInt($(row).attr('data-tnlevel'));
        if (rowLevel > 0) {
          openLevel = rowLevel - 1; // level we are looking for parent node in
        }
        rowId = $(row).attr("id");
        openList.push(row);
        // Trigger opening of this node if closed
        if ($(row).hasClass("tn-closed")) {
          // Trigger this treenode to open
          action = true;
          this._opt.debug > 1 && console.log("TreeNav.openParent opening rowLevel: " + rowLevel + " id: " + rowId);
          //.addClass('maxlevel' + (rowLevel + @_opt.openlevels))
          $(row).addClass('tn-opening').removeClass('tn-closed');
        }
      }
      if (action) {
        $(openList).each((i, el) => {
          return this._opt.debug > 1 && console.log("TreeNav.openParent open node " + $(el).attr('id'));
        });
        this.autoClose(openList);
        return this.actionOC(); // trigger action that opens or closes part of tree
      }
    }

    autoClose(openList) {
      var el, results, rowId, rowLevel;
      // given a list of treenode elements that should be open, close all other elements at the same tn-level
      if (this._opt.autoClose) {
        results = [];
        while (el = openList.pop()) {
          rowLevel = parseInt($(el).attr('data-tnlevel'));
          rowId = $(el).attr("id");
          results.push($('.treenav .treenode.tn-open[data-tnlevel="' + rowLevel + '"][id!="' + rowId + '"]').addClass("tn-closing").removeClass("tn-open"));
        }
        return results;
      }
    }

    openclose(e) {
      var el, href, row, rowLevel;
      // Trigger the opening or closing of a branch of the tree
      this._opt.debug > 0 && console.log("TreeNav.openclose " + e.type);
      if (e.type !== "click") {
        this._opt.debug > 1 && console.log("TreeNav.openclose type=" + e.type + "abandoned"); // ignore touch events
        return;
      }
      el = e.target;
      row = $(el).closest('.treerow'); // make sure we are dealing with treerow, not a child element
      if ($(row).hasClass('tn-open')) { // currently open, set to closing
        $(row).addClass('tn-closing').removeClass('tn-open');
        return this.actionOC();
      } else if ($(row).hasClass('tn-closed')) { // currently closed, set to opening
        this._opt.debug > 1 && console.log("TreeNav.openclose opening closed branch");
        rowLevel = parseInt($(row).attr('data-tnlevel'));
        //.addClass('maxlevel' + (rowLevel + @_opt.openlevels))
        $(row).addClass('tn-opening').removeClass('tn-closed');
        this.autoClose([row]); // close other open nodes at this level
        if ($(row).hasClass('tn-load') && this._opt.fragmenty !== null) {
          // if need to load tree branch then go and do this
          href = $(row).attr("data-branch");
          if (href) {
            $('body').addClass('wait');
            $.ajax({
              type: 'GET',
              url: href
            }).done((content, status, xhr) => {
              this._opt.debug > 1 && console.log("TreeNav.openclose branch load done");
              this._opt.fragmenty.append(content, status, xhr);
              $(row).removeClass("tn-load");
              return this.actionOC(); // need to trigger opening if just loaded tree branch
            }).fail(() => {
              this._opt.debug > 1 && console.log("TreeNav.openclose branch load failed");
              return $(row).addClass("tn-load tn-closed").removeClass("tn-opening");
            });
            return this._opt.debug > 1 && console.log("TreeNav.openclose loading branch: " + href);
          }
        } else {
          // branch already exists, begin opening process straight away
          return this.actionOC(); // trigger action that opens or closes part of tree
        }
      }
    }

    actionLevel() {
      var i, level;
      level = parseInt($(this).attr('data-tnlevel'));
      this._opt.debug > 1 && console.log("TreeNav.actionlevel tn-level: " + level);
      $('.treenode.tn-open[data-tnlevel="' + level + '"]').removeClass('tn-open').addClass('tn-closing');
      if (level > 0) {
        i = 0;
        while (i < level) {
          $('.treenode.tn-closed.tn-level' + i).removeClass('tn-closed').addClass('tn-opening');
          i++;
        }
      }
      this.actionOC;
    }

    actionOC() {
      var c, carryon;
      // control repeated call of action function until not needed        
      //console.log "Fragmenty actionOC"
      carryon = this.action();
      if (carryon) {
        // If true returned then keep going 
        c = window.setTimeout(this.actionOC, 25);
      }
    }

    action() {
      var carryon, closing, opening;
      // Look for treenodes with class 'tn-closing' or 'tn-opening' and
      // hide the next row due to be hidden. Returns true if an action was
      // carried out which indicates that this may need to be called again.
      // If false is returned, then opening/closing action can be deemed complete.
      carryon = false; // Assume we won't have to run again
      //console.log "Fragmenty action"

      // Act on closing branches 
      closing = $(".treenode.tn-closing");
      closing.each((index, el) => {
        var next, nextLevel, re, rowLevel;
        re = /tn-level(\d+)/;
        rowLevel = parseInt($(el).attr("data-tnlevel"));
        this._opt.debug > 1 && console.log("TreeNav.action tn-level: " + rowLevel);
        next = $(el).nextAll().not(".treehide").first(); // next unhidden row
        nextLevel = parseInt($(next).attr("data-tnlevel") || 0); // if no more elements this nextLevel = 0
        
        // If no rows left or next row is same level as closing row then finished closing 
        if (nextLevel <= rowLevel) {
          // no more elements, finished closing 
          this._opt.debug > 1 && console.log("TreeNav.action find open icon: i." + this._opt.openClass);
          $(el).removeClass("tn-closing").addClass("tn-closed").find("i." + this._opt.openClass).removeClass('twisty-open-hvr').addClass(this._opt.closedClass);
        } else {
          // carry on closing 
          if ($(next).hasClass("tn-open")) {
            
            // don't hide open nodes, close them first 
            $(next).addClass("tn-closing").removeClass("tn-open");
          } else {
            if (!$(next).hasClass("tn-closing")) {
              
              // not open and not closing, so hide 
              $(next).addClass("treehide");
            }
          }
          carryon = true;
        }
      });
      // Act on opening branches 
      opening = $(".treenode.tn-opening");
      opening.each((index, el) => {
        var endofsublevel, maxLevel, next, nextLevel, rowLevel, tryagain, waitforopen;
        
        // Find level of this row 
        rowLevel = parseInt($(el).attr("data-tnlevel"));
        maxLevel = parseInt($(el).attr("data-tnmaxlevel"));
        
        // Look for next hidden sibling at one level down from this element 
        next = $(el).next();
        tryagain = false;
        endofsublevel = false;
        waitforopen = false;
        while (true) {
          // Find level of this next element, if no next element level is 0
          nextLevel = parseInt($(next).attr("data-tnlevel") || 0);
          this._opt.debug > 1 && console.log("TreeNav.action loop nextLevel: " + nextLevel);
          if (nextLevel <= rowLevel) {
            // finished looking through branch's lower levels 
            endofsublevel = true;
          } else if ($(next).hasClass("tn-opening")) {
            waitforopen = true;
            tryagain = false; // Don't look beyond node while waiting for it to open
          } else if ((nextLevel > rowLevel + 1) || (!$(next).hasClass("treehide"))) {
            // If next element is below correct level or not hidden then skip to one after 
            next = $(next).next(); // move to next element
            tryagain = true; // moved to next element so repeat checks
          } else {
            // We have the next element we need 
            tryagain = false;
          }
          if (!(tryagain && !endofsublevel)) {
            break;
          }
        }
        if (endofsublevel) { // finished opening
          // only works for compact html as string based replace
          this._opt.debug > 1 && console.log("TreeNav.action find closed icon: i." + this._opt.closedClass);
          //.removeClass("tn-maxlevel" + maxLevel)
          $(el).removeClass("tn-opening").addClass("tn-open").find("i." + this._opt.closedClass).removeClass('twisty-closed-hvr').addClass(this._opt.openClass);
        } else if (waitforopen) {
          // Do nothing while next element is still expanding 
          carryon = true;
        } else {
          // unhide this next element and carry on (run again) 
          if (nextLevel === rowLevel + 1) {
            $(next).removeClass("treehide");
          }
          if (($(next).hasClass("tn-closed")) && (nextLevel < maxLevel)) {
            $(next).addClass("tn-opening").addClass("tn-maxlevel" + maxLevel).removeClass("tn-closed");
          }
          carryon = true;
        }
      });
      return carryon;
    }

  };

}).call(this);

//# sourceMappingURL=treenav.js.map
